# coding=utf-8
from django.utils.encoding import smart_unicode
import lxml.builder as lb
from djangodav import utils

WebDAV_NS = utils.WEBDAV_NS
WebDAV_MAP = utils.WEBDAV_NSMAP

WebDAV = utils.D


CalDAV_NS = "urn:ietf:params:xml:ns:caldav"
CalDAV_MAP = {'C': CalDAV_NS}

CalDAV = lb.ElementMaker(namespace=CalDAV_NS, nsmap=CalDAV_MAP)


CalendarServer_NS = "http://calendarserver.org/ns/"
CalendarServer_MAP = {'CS': CalendarServer_NS}

CalendarServer = lb.ElementMaker(namespace=CalendarServer_NS, nsmap=CalendarServer_MAP)

url_join = utils.url_join


class iCalendar(object):

    @staticmethod
    def unicode(string_like):
        """
        Converts vStringLike iCalendar object  (f.e. vText) into unicode string
        """
        return u"{unicode}".format(unicode=smart_unicode(string_like))

    @staticmethod
    def datetime(datetime_like):
        """
        Converts vDatetimeLike iCalendar object (f.e. vDDDTypes) into timezone-aware datetime
        """
        return datetime_like.dt


# coding=utf-8
from django.db import models
from django.utils.timezone import now
from django_caldav.utils import iCalendar


class BaseDavModel(models.Model):
    name = models.CharField(max_length=255)
    created = models.DateTimeField(default=now)
    modified = models.DateTimeField(default=now)

    class Meta(object):
        abstract = True


class CollectionModel(BaseDavModel):
    parent = models.ForeignKey('self', blank=True, null=True)
    size = 0

    class Meta(object):
        unique_together = (('parent', 'name'),)


class ObjectModel(BaseDavModel):
    parent = models.ForeignKey(CollectionModel, blank=True, null=True)
    size = models.IntegerField(default=0)
    content = models.TextField(default=u"")

    class Meta(object):
        unique_together = (('parent', 'name'),)


class CalDavEvent(models.Model):
    uid = None  # This value is not store in DB and contains value returned from item_guid
    title = models.CharField(max_length=70)

    start_datetime = models.DateTimeField()
    end_datetime = models.DateTimeField(null=True, blank=True)

    location = models.CharField(max_length=140, null=True, blank=True)

    description = models.TextField(null=True, blank=True)

    def finalize(self):
        """
        Converts from iCalendar types to python types
        """
        if self.uid:
            self.uid = iCalendar.unicode(self.uid)

        self.title = iCalendar.unicode(self.title)

        self.start_datetime = iCalendar.datetime(self.start_datetime).replace(tzinfo=None)

        if self.end_datetime:
            self.end_datetime = iCalendar.datetime(self.end_datetime).replace(tzinfo=None)

        if self.location:
            self.location = iCalendar.unicode(self.location)

        if self.description:
            self.description = iCalendar.unicode(self.description)

    class Meta(object):
        abstract = True


__author__ = 'Petr Knap <knap@wpj.cz>'


def print_xml(xml_as_string):
    if xml_as_string:
        from xml.dom import minidom
        xml_as_dom = minidom.parseString(xml_as_string)
        print xml_as_dom.toprettyxml()